'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.debug = exports.Handlebars = exports.HandlebarsHelpers = exports.NerveHelpers = exports.NerveActiveUser = exports.Module = exports.NerveApi = exports.NervePage = exports.NerveApp = exports.NerveObject = void 0;
const _ = require("lodash");
const NerveObject = require("./object");
exports.NerveObject = NerveObject;
const fs = require("fs");
const uuid = require("node-uuid");
const express = require("express");
const cookieParser = require("cookie-parser");
const Router = require("./router");
const NervePage = require("./page");
exports.NervePage = NervePage;
const NerveApi = require("./api");
exports.NerveApi = NerveApi;
const Module = require("./module");
exports.Module = Module;
const NerveActiveUser = require("./active-user");
exports.NerveActiveUser = NerveActiveUser;
const handlebarsHelpers = require("./lib/handlebars-helpers/main");
exports.HandlebarsHelpers = handlebarsHelpers;
const NerveHelpers = require("./lib/helpers");
exports.NerveHelpers = NerveHelpers;
const debug = require("./lib/debug");
exports.debug = debug;
const locales = require("./lib/locales");
const NerveProperty = require("./lib/property");
const Handlebars = require("handlebars");
exports.Handlebars = Handlebars;
class NerveApp extends NerveObject {
    constructor() {
        super();
        this.JS_VERSIONS = {};
        this.CSS_VERSIONS = {};
        this.SPRITE_VERSIONS = {};
        this.isReady = false;
        this.routes = {};
        this.publicRoutes = {};
        this.config = {};
        this.allConfig = {};
        this.server = express();
        this.server.enable('trust proxy');
        this.server.enable('case sensitive routing');
        this.server.enable('strict routing');
        this.server.disable('x-powered-by');
        this.server.disable('etag');
        this.server.use((req, res, next) => {
            req.id = uuid.v4();
            next();
        });
        this.server.use(cookieParser());
        this.router = new Router(this);
        this.environment = process.env.NODE_NERVE_ENV || 'dev';
        setTimeout(() => {
            this.readCfg()
                .then(() => {
                debug.setLevel(this.getCfg('logLevel'));
                locales.init(this);
                if (this.getCfg('localStatic')) {
                    this.server.use(express.static(this.getCfg('localStatic')));
                }
                this.isReady = true;
                this.emit('ready');
            })
                .catch((err) => debug.error(err.toString()));
        });
    }
    getServer() {
        return this.server;
    }
    getRouter() {
        return this.router;
    }
    getJsVersions() {
        return this.JS_VERSIONS;
    }
    getCssVersions() {
        return this.CSS_VERSIONS;
    }
    getSpriteVersions() {
        return this.SPRITE_VERSIONS;
    }
    listen(port, host, callback) {
        this.connection = this.server.listen(port, host, callback);
        return this.connection;
    }
    close() {
        this.connection.close();
    }
    route(routes) {
        if (routes.public) {
            this.routes = _.assign(this.routes, routes.public);
            this.publicRoutes = _.assign(this.publicRoutes, routes.public);
        }
        if (routes.protected) {
            this.routes = _.assign(this.routes, routes.protected);
        }
        if (!routes.public && !routes.protected) {
            this.routes = _.assign(this.routes, routes);
            this.publicRoutes = _.assign(this.publicRoutes, routes);
        }
        Object.keys(this.routes).forEach((url) => this.router.get(url, this.routes[url]));
    }
    env(env) {
        this.environment = env;
        if (this.allConfig) {
            this.config = this.allConfig[this.environment];
        }
    }
    readCfg() {
        return new Promise((resolve, reject) => {
            fs.readFile('./nerve.json', (err, content) => {
                if (err) {
                    reject(err);
                    throw err;
                }
                this.allConfig = JSON.parse(content.toString());
                this.config = this.allConfig[this.environment];
                resolve(this.config);
            });
        });
    }
    setCfg(key, value) {
        NerveProperty.set(this.config, key, value);
        return this;
    }
    getCfg(key) {
        return NerveProperty.get(this.config, key);
    }
    getPublicRoutes() {
        let routes = [];
        Object.keys(this.publicRoutes).forEach((item) => {
            routes.push({
                key: item,
                route: this.publicRoutes[item].replace(/^src\//, '')
            });
        });
        return routes;
    }
    setDebugLevel(level) {
        debug.setLevel(level);
    }
    setWorkerIndex(index) {
        this.workerIndex = index;
    }
    ready() {
        return new Promise((resolve) => {
            if (this.isReady) {
                resolve();
            }
            else {
                this.on('ready', resolve);
            }
        });
    }
}
exports.NerveApp = NerveApp;
handlebarsHelpers(Handlebars);
//# sourceMappingURL=app.js.map